<?php

namespace Drupal\miniorange_saml\Form;

use Drupal\Core\Url;
use Drupal\Core\Form\FormBase;
use Drupal\miniorange_saml\MiniorangeSamlConstant;
use Drupal\miniorange_saml\Utilities;
use Drupal\Core\Form\FormStateInterface;

/**
 * Form for customer setup in miniOrange SAML.
 */
class MiniorangeSamlCustomerSetup extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'miniorange_saml_customer_setup';
  }

  /**
   * Builds the CustomerSetup form.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return array
   *   The form array
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $current_status = \Drupal::config('miniorange_saml.settings')->get('miniorange_saml_status');

    $form['#attached']['library'][] = 'miniorange_saml/miniorange_saml.admin';
    $form['#attached']['library'][] = 'core/drupal.dialog.ajax';

    if (empty($current_status) || $current_status == 'CUSTOMER_SETUP') {
      $form['div_for_start'] = [
        '#markup' => t('<div class="mo_saml_sp_main_container"><div class="mo_saml_sp_container">'),
      ];

      $form['markup_14'] = [
        '#markup' => $this->t('<div class="mo_saml_font_for_heading">Register/Login</div><p style="clear: both"></p><hr>'),
      ];

      $form['mo_saml_login_tab'] = [
        '#type' => 'fieldset',
        '#title' => $this->t('Please login with your miniOrange account.'),
        '#attributes' => ['style' => 'padding:2% 2% 5%; margin-bottom:2%'],
      ];

      $form['mo_saml_login_tab']['miniorange_saml_customer_login_username'] = [
        '#type' => 'email',
        '#title' => $this->t('Email'),
        '#required' => TRUE,
      ];

      $form['mo_saml_login_tab']['miniorange_saml_customer_login_password'] = [
        '#type' => 'password',
        '#title' => $this->t('Password'),
        '#required' => TRUE,
      ];

      $form['mo_saml_login_tab']['miniorange_saml_customer_login_button'] = [
        '#type' => 'submit',
        '#value' => $this->t('Login'),
        '#button_type' => 'primary',
        '#prefix' => '<div class="container-inline">',
      ];

      $form['mo_saml_login_tab']['register_link'] = [
        '#type' => 'link',
        '#title' => $this->t('Create an account'),
        '#url' => Url::fromUri(MiniorangeSamlConstant::CREATE_ACCOUNT_URL),
        '#attributes' => ['class' => 'button', 'target' => '_blank'],
        '#suffix' => '</div></div>',
      ];
    }
    elseif ($current_status == 'PLUGIN_CONFIGURATION') {

      $form['markup_top'] = [
        '#markup' => t('<div class="mo_saml_sp_main_container"><div class="mo_saml_sp_container">
                                  <div class="mo_saml_welcome_message">Thank you for registering with miniOrange</div>'),
      ];

      /* Create container to hold @moProfileDetails form elements. */
      $form['mo_saml_profile_details_tab'] = [
        '#type' => 'fieldset',
        '#title' => $this->t('Profile Details:'),
        '#attributes' => ['style' => 'padding:2% 2% 6%; margin-bottom:2%'],
      ];

      $module_info = \Drupal::service('extension.list.module')->getExtensionInfo('miniorange_saml');

      $header = [t('Attribute'), t('Value')];
      $options = [
            ['Customer Email', \Drupal::config('miniorange_saml.settings')->get('miniorange_saml_customer_admin_email')],
            ['Customer ID', \Drupal::config('miniorange_saml.settings')->get('miniorange_saml_customer_id')],
            ['Module Version', $module_info['version']],
            ['Drupal Version', Utilities::miniorangeGetDrupalCoreVersion()],
            ['PHP Version', phpversion()],
      ];

      $form['mo_saml_profile_details_tab']['fieldset']['customerinfo'] = [
        '#theme' => 'table',
        '#header' => $header,
        '#rows' => $options,
        '#prefix' => '<br>',
        '#attributes' => ['style' => 'margin:1% 0% 7%;'],
      ];

      $form['mo_saml_profile_details_tab']['miniorange_saml_customer_Remove_Account_info'] = [
        '#markup' => t('<br/><h4>Remove Account:</h4><p>This section will help you to remove your current
                        logged in account without losing your current configurations.</p>'),
      ];

      $form['mo_saml_profile_details_tab']['miniorange_saml_customer_Remove_Account'] = [
        '#type' => 'link',
        '#title' => $this->t('Remove Account'),
        '#url' => Url::fromRoute('miniorange_saml.modal_form'),
        '#attributes' => [
          'class' => [
            'use-ajax',
            'button',
          ],
        ],
        '#suffix' => '</div>',
      ];

      $form['#attached']['library'][] = 'core/drupal.dialog.ajax';

      Utilities::advertiseNetworkSecurity($form, $form_state);

      return $form;
    }

    Utilities::advertiseNetworkSecurity($form, $form_state);

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $form_values = $form_state->getValues();

    $username = trim($form_values['miniorange_saml_customer_login_username']);
    $password = trim($form_values['miniorange_saml_customer_login_password']);

    Utilities::customerSetupSubmit($username, '', $password, TRUE);
  }

}
