<?php

namespace Drupal\miniorange_saml;

use Drupal\miniorange_saml\Api\MoAuthApi;

/**
 * The MiniorangeSAMLCustomer class.
 */
class MiniorangeSamlCustomer {
  /**
   * Customer email.
   *
   * @var string|null
   */
  public $email;

  /**
   * Customer phone.
   *
   * @var string|null
   */
  public $phone;

  /**
   * Customer password.
   *
   * @var string|null
   */
  public $password;

  /**
   * OtpToken.
   *
   * @var string|null
   */
  public $otpToken;

  /**
   * Constructor.
   */
  public function __construct($email, $phone, $password, $otp_token) {
    $this->email = $email;
    $this->phone = $phone;
    $this->password = $password;
    $this->otpToken = $otp_token;
  }

  /**
   * Check if customer exists.
   */
  public function checkCustomer() {
    $url = MiniorangeSamlConstant::CUSTOMER_CHECK_API;
    $fields = ['email' => $this->email];
    $api = new MoAuthApi();
    return $api->makeCurlCall($url, $fields);
  }

  /**
   * Get Customer Keys.
   */
  public function getCustomerKeys() {
    $url = MiniorangeSamlConstant::CUSTOMER_GET_KEYS;
    $fields = ['email' => $this->email, 'password' => $this->password];
    $api = new MoAuthApi();
    return $api->makeCurlCall($url, $fields);
  }

  /**
   * Send OTP.
   */
  public function sendOtp() {
    $url = MiniorangeSamlConstant::AUTH_CHALLENGE_API;
    $username = \Drupal::config('miniorange_saml.settings')->get('miniorange_saml_customer_admin_email');
    $fields = [
      'customerKey' => MiniorangeSamlConstant::DEFAULT_CUSTOMER_ID,
      'email' => $username,
      'authType' => 'EMAIL',
    ];
    $api = new MoAuthApi();
    $header = $api->getHttpHeaderArray();
    return $api->makeCurlCall($url, $fields, $header);
  }

}
